#!/usr/bin/perl -w

# $Id: pmgraph.pl,v 1.10 2005/09/15 16:59:46 andreaso Exp $

use strict;
use Fcntl;
use File::Copy;
use File::Path;
use File::Spec;
use File::Temp 'tempdir';
use RRDs;
use POSIX qw(strftime);

sub create_rrd;
sub update_rrd;
sub generate_gfx;
sub catch_sigint;

# The graphs to include in the HTML page.
# 1 means to include it, 0 means not to.
my %graphs_to_create = (
    drops          => 1,
    mbit_per_sec   => 1,
    alerts_per_sec => 1,
    kpackets       => 1,
    bytes_per_pkt  => 1,
    syn_stats      => 1,
    session_stats  => 1,
    open_sessions  => 1,
    stream_stats   => 1,
    frags_stats    => 1,
    frag_counters  => 1,
    cpu_stats      => 1,
);

# Options you may want to change.
my %options = (
    refresh_interval => 120,
    img_height       => 100,
    img_width        => 1000,
    img_format       => 'PNG',
    img_suffix       => 'png',
);

my $VERSION = 'pmgraph v0.1 by Andreas stling <andreaso@it.su.se>';

my $USAGE = << "USAGE";
$VERSION

Usage: $0 <output directory> <perfmon file> [number of interfaces]

<output directory> is where to put the resulting images and perfstats.html
<perfmon file> is the ascii file created by perfmonitor (\"-\" for standard input)

[number of interfaces] must be specified only if <perfmon file> was created
on a sensor with multiple instances of Snort listening on separate interfaces
and logging to this file at the same time. See the README for more information.

USAGE


my $output_dir     = shift || die($USAGE);
my $perfmon_file   = shift || die($USAGE);
my $num_interfaces = shift || "1";


die("output directory \"$output_dir\" does not exist\n")
  unless (-d "$output_dir");

die("output directory \"$output_dir\" is not writable by you\n")
  unless (-w "$output_dir");

die("perfmon input file \"$perfmon_file\" does not exist\n")
  if ($perfmon_file ne "-" && !-f $perfmon_file);

my $tmpdir = tempdir("perfmon-stats.XXXXXXXXXX", DIR => File::Spec->tmpdir())
  or die("could not create temporary directory: $!\n");

my $rrd_db             = "$tmpdir/temp.rrd";
my $fixed_perfmon_file = "$tmpdir/perfmon.fixed";


$SIG{__DIE__} = sub
{
    print STDERR "Error: $_[0]";
    rmtree("$tmpdir", 0, 1);
    exit(1);
};

$SIG{INT} = \&catch_sigint;

my %perfmon_fluff = (
    num_interfaces => $num_interfaces
);


preprocess_perfmon_file($perfmon_file, $fixed_perfmon_file, \%perfmon_fluff);
create_rrd($rrd_db, $fixed_perfmon_file, \%perfmon_fluff);
update_rrd($rrd_db, $fixed_perfmon_file, \%perfmon_fluff);
generate_gfx($rrd_db, $fixed_perfmon_file, $output_dir, \%perfmon_fluff);

rmtree("$tmpdir", 0, 1);



sub create_rrd
{
    my $rrd_db            = shift;
    my $perfmon_file      = shift;
    my $perfmon_fluff_ref = shift;

    my $interval = $$perfmon_fluff_ref{interval};

    print "Creating temporary RRD database.\n";

    my $cmd = << '_EOD';

      local $SIG{'__DIE__'};

      RRDs::create (
        "$rrd_db",
        "--start=" . ($$perfmon_fluff_ref{start_time} - 1),
        "--step=$interval",
        "DS:drops:GAUGE:$interval:U:U",
        "DS:Mbit_wire:GAUGE:$interval:U:U",
        "DS:Mbit_wire_frag:GAUGE:$interval:U:U",
        "DS:Mbit_appl:GAUGE:$interval:U:U",
        "DS:alerts:GAUGE:$interval:U:U",
        "DS:kpackets_total:GAUGE:$interval:U:U",
        "DS:kpackets_fragmented:GAUGE:$interval:U:U",
        "DS:avg_bytes:GAUGE:$interval:U:U",
        "DS:syn_stat_syn:GAUGE:$interval:U:U",
        "DS:syn_stat_syn_ack:GAUGE:$interval:U:U",
        "DS:session_stat_new:GAUGE:$interval:U:U",
        "DS:session_stat_del:GAUGE:$interval:U:U",
        "DS:session_stat_open:GAUGE:$interval:U:U",
        "DS:session_stat_max:GAUGE:$interval:U:U",
        "DS:stream_stat_flushes:GAUGE:$interval:U:U",
        "DS:stream_stat_faults:GAUGE:$interval:U:U",
        "DS:frag_stat_creates:GAUGE:$interval:U:U",
        "DS:frag_stat_compl:GAUGE:$interval:U:U",
        "DS:frag_stat_inserts:GAUGE:$interval:U:U",
        "DS:frag_stat_del:GAUGE:$interval:U:U",
        "DS:frag_stat_flushes:GAUGE:$interval:U:U",
        "DS:frag_stat_timeouts:GAUGE:$interval:U:U",
        "DS:frag_stat_faults:GAUGE:$interval:U:U",

_EOD

    for my $i (1 .. $$perfmon_fluff_ref{num_cpus}) {
         $cmd .= "\"DS:cpu${i}_stat_user:GAUGE:$interval:U:U\",\n".
                 "\"DS:cpu${i}_stat_sys:GAUGE:$interval:U:U\",\n".
                 "\"DS:cpu${i}_stat_idle:GAUGE:$interval:U:U\",\n";
    }

    $cmd .= << '_EOD';
      "RRA:AVERAGE:0.5:1:" . $$perfmon_fluff_ref{num_lines});
       my $err=RRDs::error;
       die("RRD error: $err\n") if ($err);
_EOD

    eval $cmd;
    die($@) if ($@);
}



sub update_rrd
{
    my $rrd_db            = shift;
    my $perfmon_file      = shift;
    my $perfmon_fluff_ref = shift;

    my $num_cpus = $$perfmon_fluff_ref{num_cpus};

    print "Inserting values into temporary RRD database.\n";

    open(PERF, $perfmon_file)
      or die("could not open $perfmon_file for reading: $!\n");

    while (chomp(my @fields = split(/,/, <PERF>))) {

        my $time                 = $fields[0];
        my $drops                = $fields[1];
        my $mbit_wire            = $fields[30];
        my $mbit_wire_frag       = $fields[31];
        my $mbit_appl            = $fields[34];
        my $alerts               = $fields[3];
        my $kpackets             = $fields[4];
        my $kpackets_fragmented  = $fields[41];
        my $avg_bytes            = $fields[5];

        my %syn_stat = (
            syn       => $fields[7],
            syn_ack   => $fields[8],
        );

        my %session_stat = (
            new       => $fields[9],
            del       => $fields[10],
            open      => $fields[11],
            max       => $fields[12],
        );

        my %stream_stat = (
            flushes   => $fields[13],
            faults    => $fields[14],
        );

        my %frag_stat = (
            compl     => $fields[17],
            inserts   => $fields[18],
            del       => $fields[19],
            flushes   => $fields[21],
            creates   => $fields[16],
            timeouts  => $fields[24],
            flushes   => $fields[25],
        );

        my %cpu_stat;
        my $field_cnt = 0;
        for my $i (1 .. $num_cpus) {
            $cpu_stat{$i}{user} = $fields[27 + $field_cnt];
            $cpu_stat{$i}{sys}  = $fields[27 + $field_cnt + 1];
            $cpu_stat{$i}{idle} = $fields[27 + $field_cnt + 2];
            $field_cnt += 3;
        }

        my $cpu_template_string;
        for my $i (1 .. $num_cpus) {
            $cpu_template_string .= "cpu${i}_stat_user:cpu${i}_stat_sys:cpu${i}_stat_idle:";
        }

        $cpu_template_string =~ s/:$//;

        my $template =
          "--template=drops:Mbit_wire:Mbit_wire_frag:Mbit_appl:alerts:kpackets_total:kpackets_fragmented:avg_bytes:".
          "syn_stat_syn:syn_stat_syn_ack:".
          "session_stat_new:session_stat_del:session_stat_open:session_stat_max:".
          "stream_stat_flushes:stream_stat_faults:".
          "frag_stat_creates:frag_stat_compl:frag_stat_inserts:frag_stat_del:frag_stat_flushes:".
          "frag_stat_timeouts:frag_stat_flushes:".
          $cpu_template_string;

        my $values =
          "$time:$drops:$mbit_wire:$mbit_wire_frag:$mbit_appl:$alerts:$kpackets:$kpackets_fragmented:$avg_bytes:".
          "$syn_stat{syn}:$syn_stat{syn_ack}:".
          "$session_stat{new}:$session_stat{del}:$session_stat{open}:$session_stat{max}:".
          "$stream_stat{flushes}:$stream_stat{faults}:".
          "$frag_stat{creates}:$frag_stat{compl}:$frag_stat{inserts}:$frag_stat{del}:$frag_stat{flushes}:".
          "$frag_stat{timeouts}:$frag_stat{flushes}:";

        for my $i (1 .. $num_cpus) {
          $values .= "$cpu_stat{$i}{user}:$cpu_stat{$i}{sys}:$cpu_stat{$i}{idle}:"
        }

        $values =~ s/:$//;

        RRDs::update($rrd_db, $template, $values);
        my $err=RRDs::error;
        die("RRD error: $err\n") if ($err);
    }

    close(PERF);
}



sub generate_gfx
{
    my $rrd_db            = shift;
    my $perfmon_file      = shift;
    my $output_dir        = shift;
    my $perfmon_fluff_ref = shift;

    my $start_time = $$perfmon_fluff_ref{start_time};
    my $end_time   = $$perfmon_fluff_ref{end_time};
    my $out_file   = "$output_dir/perfstats.html";
    my $err;

    my $tmp_out_file = "$tmpdir/perfstats.html.tmp";

    print "Generating images and $out_file.\n";

  # Generate HTML page with images.
    open(HTML, ">$tmp_out_file")
      or die("could not open $tmp_out_file for writing: $!\n");

    my $curtime = strftime("%Y-%m-%d %H:%M:%S (%Z)", localtime);
    my $stime   = strftime("%Y-%m-%d %H:%M:%S (%Z)", localtime($start_time));
    my $etime   = strftime("%Y-%m-%d %H:%M:%S (%Z)", localtime($end_time));

    print HTML << "HTMLSTUFF";
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN">
<html>
  <head>
    <title>Perfmonitor stats $curtime</title>
    <meta http-equiv="refresh" content="$options{refresh_interval}">
  </head>
  <body>

  <center>
  <table bgcolor=\"#F0F0F0\">
    <tr><td>Graphs generated:          <td>$curtime
    <tr><td>First entry in input file: <td>$stime
    <tr><td>Last entry in input file:  <td>$etime
  </table>
  </center>

  <br>
  <center>

HTMLSTUFF


  # Used in all graphs.
    my @common_cfg = (
      "--imgformat=$options{img_format}",
      "--height=$options{img_height}",
      "--width=$options{img_width}",
      "--start=$start_time",
      "--end=$end_time",
      "--lower-limit=0",
    );


  # % drops.
    if ($graphs_to_create{drops}) {
        RRDs::graph ("$output_dir/drops.$options{img_suffix}", @common_cfg,
            "--title=Dropped packets (%)",
            "--upper-limit=1",
            "DEF:drops=$rrd_db:drops:AVERAGE",
            "AREA:drops#FF0000"
        );

        $err=RRDs::error;
        die("RRD error: $err\n") if ($err);
        print HTML "    <img src=\"drops.$options{img_suffix}\"><br>\n";
    }


  # Mbit/s stats
    if ($graphs_to_create{mbit_per_sec}) {
        RRDs::graph ("$output_dir/Mbit.$options{img_suffix}", @common_cfg,
            "--title=Mbit per second",
            "DEF:Mbit=$rrd_db:Mbit_wire:AVERAGE",
            "DEF:Mbit_appl=$rrd_db:Mbit_appl:AVERAGE",
            "DEF:Mbit_wire_frag=$rrd_db:Mbit_wire_frag:AVERAGE",
            "AREA:Mbit#00FF00:Total on wire",
            "AREA:Mbit_appl#0000FF:Application layer",
            "AREA:Mbit_wire_frag#FF0000:fragmented on wire",
        );

        $err=RRDs::error;
        die("RRD error: $err\n") if ($err);
        print HTML "    <img src=\"Mbit.$options{img_suffix}\"><br>\n";
    }


  # alerts/sec
    if ($graphs_to_create{alerts_per_sec}) {
        RRDs::graph ("$output_dir/alerts.$options{img_suffix}", @common_cfg,
            "--title=Alerts per second",
            "--upper-limit=1",
            "DEF:alerts=$rrd_db:alerts:AVERAGE",
            "AREA:alerts#FF0000"
        );

        $err=RRDs::error;
        die("RRD error: $err\n") if ($err);
        print HTML "    <img src=\"alerts.$options{img_suffix}\"><br>\n";
    }


  # kpackets.
    if ($graphs_to_create{kpackets}) {
        RRDs::graph ("$output_dir/kpackets.$options{img_suffix}", @common_cfg,
            "--title=Kpackets per second",
            "DEF:kpackets_total=$rrd_db:kpackets_total:AVERAGE",
            "DEF:kpackets_fragmented=$rrd_db:kpackets_fragmented:AVERAGE",
            "AREA:kpackets_total#0000FF:total",
            "AREA:kpackets_fragmented#FF0000:fragmented",
        );

        $err=RRDs::error;
        die("RRD error: $err\n") if ($err);
        print HTML "    <img src=\"kpackets.$options{img_suffix}\"><br>\n";
    }


  # average bytes/packet.
    if ($graphs_to_create{bytes_per_pkt}) {
        RRDs::graph ("$output_dir/avg_bytes.$options{img_suffix}", @common_cfg,
            "--title=Average bytes per packet",
            "DEF:avg_bytes=$rrd_db:avg_bytes:AVERAGE",
            "AREA:avg_bytes#0000FF"
        );

        $err=RRDs::error;
        die("RRD error: $err\n") if ($err);
        print HTML "    <img src=\"avg_bytes.$options{img_suffix}\"><br>\n";
    }


  # syn stats
    if ($graphs_to_create{syn_stats}) {
        RRDs::graph ("$output_dir/syn_stats.$options{img_suffix}", @common_cfg,
            "--title=SYN + SYN/ACK packets per second",
            "DEF:syn_stat_syn=$rrd_db:syn_stat_syn:AVERAGE",
            "DEF:syn_stat_syn_ack=$rrd_db:syn_stat_syn_ack:AVERAGE",
            "AREA:syn_stat_syn#0000FF:SYN",
            "AREA:syn_stat_syn_ack#FF0000:SYN/ACK"
        );

        $err=RRDs::error;
        die("RRD error: $err\n") if ($err);
        print HTML "    <img src=\"syn_stats.$options{img_suffix}\"><br>\n";
    }


  # session stats, new/deleted
    if ($graphs_to_create{session_stats}) {
        RRDs::graph ("$output_dir/session_stats.$options{img_suffix}", @common_cfg,
            "--title=Session events per second",
            "DEF:session_stat_new=$rrd_db:session_stat_new:AVERAGE",
            "DEF:session_stat_del=$rrd_db:session_stat_del:AVERAGE",
            "LINE2:session_stat_new#0000FF:new",
            "LINE1:session_stat_del#FF0000:deleted",
        );

        $err=RRDs::error;
        die("RRD error: $err\n") if ($err);
        print HTML "    <img src=\"session_stats.$options{img_suffix}\"><br>\n";
    }


  # session stats, current open
    if ($graphs_to_create{open_sessions}) {
        RRDs::graph ("$output_dir/current_session_stats.$options{img_suffix}", @common_cfg,
            "--title=Open sessions",
            "DEF:session_stat_open=$rrd_db:session_stat_open:AVERAGE",
            "DEF:session_stat_max=$rrd_db:session_stat_max:AVERAGE",
            "LINE2:session_stat_open#0000FF:open sessions",
            "LINE2:session_stat_max#00FF00:max sessions",
        );

        $err=RRDs::error;
        die("RRD error: $err\n") if ($err);
        print HTML "    <img src=\"current_session_stats.$options{img_suffix}\"><br>\n";
    }


  # stream stats
    if ($graphs_to_create{stream_stats}) {
        RRDs::graph ("$output_dir/stream_stats.$options{img_suffix}", @common_cfg,
            "--title=Stream events per second",
            "DEF:stream_stat_flushes=$rrd_db:stream_stat_flushes:AVERAGE",
            "DEF:stream_stat_faults=$rrd_db:stream_stat_faults:AVERAGE",
            "AREA:stream_stat_flushes#FF00FF:flushes",
            "AREA:stream_stat_faults#FF0000:faults",
        );

        $err=RRDs::error;
        die("RRD error: $err\n") if ($err);
        print HTML "    <img src=\"stream_stats.$options{img_suffix}\"><br>\n";
    }


  # frag events/sec
    if ($graphs_to_create{frags_stats}) {
        RRDs::graph ("$output_dir/frag_events.$options{img_suffix}", @common_cfg,
            "--title=frag events per second",
            "DEF:frag_stat_creates=$rrd_db:frag_stat_creates:AVERAGE",
            "DEF:frag_stat_compl=$rrd_db:frag_stat_compl:AVERAGE",
            "DEF:frag_stat_inserts=$rrd_db:frag_stat_inserts:AVERAGE",
            "DEF:frag_stat_del=$rrd_db:frag_stat_del:AVERAGE",
            "DEF:frag_stat_flushes=$rrd_db:frag_stat_flushes:AVERAGE",
            "LINE2:frag_stat_creates#FF00FF:creates",
            "LINE2:frag_stat_compl#00FF00:completes",
            "LINE2:frag_stat_inserts#0000FF:inserts",
            "LINE2:frag_stat_del#FFFF00:deletes",
            "LINE2:frag_stat_flushes#00FFFF:flushes",
        );

        $err=RRDs::error;
        die("RRD error: $err\n") if ($err);
        print HTML "    <img src=\"frag_events.$options{img_suffix}\"><br>\n";
    }


  # frag stats, counters
    if ($graphs_to_create{frags_counters}) {
        RRDs::graph ("$output_dir/frag_stats.$options{img_suffix}", @common_cfg,
            "--title=frag stats",
            "DEF:frag_stat_timeouts=$rrd_db:frag_stat_timeouts:AVERAGE",
            "DEF:frag_stat_flushes=$rrd_db:frag_stat_flushes:AVERAGE",
            "LINE2:frag_stat_timeouts#FF0000:timeouts",
            "LINE2:frag_stat_flushes#0000FF:flushes",
        );

        $err=RRDs::error;
        die("RRD error: $err\n") if ($err);
        print HTML "    <img src=\"frag_stats.$options{img_suffix}\"><br>\n";
    }


  # stats for each cpu.
    if ($graphs_to_create{cpu_stats}) {
        for my $i (1 .. $$perfmon_fluff_ref{num_cpus}) {
            RRDs::graph ("$output_dir/cpu${i}_stats.$options{img_suffix}", @common_cfg,
                "--title=CPU$i stats (%)",
                "--upper-limit=100",
                "DEF:cpu${i}_stat_user=$rrd_db:cpu${i}_stat_user:AVERAGE",
                "DEF:cpu${i}_stat_sys=$rrd_db:cpu${i}_stat_sys:AVERAGE",
                "DEF:cpu${i}_stat_idle=$rrd_db:cpu${i}_stat_idle:AVERAGE",
                "LINE2:cpu${i}_stat_user#0000FF:user",
                "LINE2:cpu${i}_stat_sys#FFA000:system ",
                "LINE2:cpu${i}_stat_idle#00FF00:idle",
            );

            $err=RRDs::error;
            die("RRD error: $err\n") if ($err);
            print HTML "    <img src=\"cpu${i}_stats.$options{img_suffix}\"><br>\n";
        }
    }

    print HTML "  </center></body>\n";
    print HTML "</html>\n";

    close(HTML);

    copy($tmp_out_file, $out_file) or die("Could not copy $tmp_out_file to $out_file: $!\n");
}



# Read perfmon info from stdin or file, aggregate entries
# if requested, and write to new file in temporary dir.
# Also collect some info about the new perfmon file.
sub preprocess_perfmon_file
{
    my $orig_perfmon_file  = shift;
    my $fixed_perfmon_file = shift;
    my $perfmon_fluff_ref  = shift;

    my $i               = 0;
    my $entries         = 0;
    my $last_num_fields = 0;
    my $line_num        = 0;
    my $num_cpus        = 0;
    my $MIN_ENTRIES     = 10;

    my (@merged_fields, $start_time, $end_time);

    my $num_interfaces = $$perfmon_fluff_ref{num_interfaces};

    my %avg_fields;


    print "Processing data from \"$orig_perfmon_file\".\n";

    print "Merging groups of $num_interfaces lines\n"
      if ($num_interfaces > 1);

    open(INFILE, $orig_perfmon_file)
      or die("could not open $orig_perfmon_file for reading: $!");

    open(OUTFILE, ">$fixed_perfmon_file")
      or die("could not open $fixed_perfmon_file for writing: $!");

    while (my $line = <INFILE>) {
        $line_num++;

      # Line may end with empty comma.
        $line =~ s/,$//;

        my @fields = split(/,/, $line);

        my $num_fields = @fields;
        next if ($num_fields < 2);

      # Expect Snort 2.4.0 format.
        my $min_fields = 47;

        die("Not a valid perfmon file: not enough comma-separated fields, got $num_fields but ".
            "expected at least $min_fields\n".
            "Make sure you run the correct versions of $0 and Snort\n")
          unless ($num_fields >= $min_fields);

      # In case someone keeps the same result file when upgrading and the format changed.
        if ($last_num_fields && $last_num_fields != $num_fields) {
            die("Line $line_num in input file contains more fields than ".
                "previous line ($num_fields vs $last_num_fields).\n".
                "Perhaps you need to start the perfmonitor preprocessor with a clean file?\n");
        }


      # Check how many CPUs we have stats for.
        unless ($num_cpus) {
            $num_cpus = $fields[26];

            unless ($num_cpus =~ /^\d+$/ && $num_cpus && $num_cpus < 17) {
                die("Incorrect/unsupported format in input file: number ".
                    "of CPUs set to $num_cpus\n");
            }

            if ($num_cpus == 1) {
                print "Got stats from $num_cpus CPU\n";
            } else {
                print "Got stats from $num_cpus CPUs\n";
            }
        }

      # 3 extra fields for each CPU.
        my $num_cpu_fields = $num_cpus * 3;

        $last_num_fields = $num_fields;

        $start_time = $fields[0] unless (defined($start_time));
        $end_time   = $fields[0];

      # Fields to divide by number of interfaces after merge to get
      # the average instead of total sum., like timestamp and stuff
      # meassured in %.
        my $field_cnt = 0;
        unless (keys(%avg_fields)) {
            $avg_fields{0} = 'timestamp';
            $avg_fields{1} = 'drops';

            for my $i (1 .. $num_cpus) {
                $avg_fields{27 + $field_cnt}     = 'cpu_user';
                $avg_fields{27 + $field_cnt + 1} = 'cpu_sys';
                $avg_fields{27 + $field_cnt + 2} = 'cpu_idle';
                $field_cnt += 3;
            }
        }

      # If there is only one interface, write the line back to the new file.
      # If there are multiple interfaces, merge the fields.
        if ($num_interfaces == 1 ) {
           $entries++;
           print OUTFILE $line;
        } else {

          # Merge all fields.
            for my $j (0 .. $#fields) {
                $merged_fields[$j] += $fields[$j];
            }

          # Check if it's time to regard current contents of
          # @merged_fields as a new entry.
            if (++$i % $num_interfaces == 0) {

              # Divide fields with number of interfaces where we need the average.
                for my $j (0 .. $#fields) {
                    $merged_fields[$j] = $merged_fields[$j]  / $num_interfaces
                      if ($avg_fields{$j});
                }

              # Timestamp must be int.
                $merged_fields[0] = int($merged_fields[0]);
                chomp(my $merged_line = join(',', @merged_fields));
                print OUTFILE "$merged_line\n";
                $#merged_fields = -1;
                $entries++;
            }
        }
    }

    close(INFILE);
    close(OUTFILE);

    die("Not enough entries in perfmon file ".
        "(required $MIN_ENTRIES but got only $entries)\n")
      unless ($entries >= $MIN_ENTRIES);

    die("Could not grab start time and end time from perfmon file\n")
       unless (defined($start_time) && defined($end_time) && $end_time > $start_time);


  # Assume time between 2nd and 3rd entries is the perfmonitor log interval in use.
  # May be wrong if pktcnt is not exceeded.
    open(FIXEDPERF, $fixed_perfmon_file)
      or die("Could not open $fixed_perfmon_file for reading: $!\n");
    $_ = <FIXEDPERF>;

    my $second_time = (split(/,/, <FIXEDPERF>))[0];
    my $third_time  = (split(/,/, <FIXEDPERF>))[0];
    close(FIXEDPERF);

    die("Could not find out time interval\n")
       unless (defined($second_time) && defined($third_time) && $third_time > $second_time);

  # Increase interval a bit or the graphs will look bad if the Snort
  # machine can't keep up with the accurate logging interval, or if
  # multi-interface machine's logging is out of sync.
    my $interval = int(($second_time - $start_time) * 1.2);


    $$perfmon_fluff_ref{num_lines}  = $entries;
    $$perfmon_fluff_ref{start_time} = $start_time;
    $$perfmon_fluff_ref{end_time}   = $end_time;
    $$perfmon_fluff_ref{interval}   = $interval;
    $$perfmon_fluff_ref{num_cpus}   = $num_cpus;
}



sub catch_sigint
{
    $SIG{INT} = 'IGNORE';
    print "\n";
    die("Signal caught, cleaning up and exiting.\n");
}
